#include <glad/glad.h>
#include <GLFW/glfw3.h>
#include <iostream>
#include <fstream>
#include <cstring>
#include <cmath>
#include <cstdlib>

#include "coil/coil.hpp"
#include "coil/segment.hpp"

#define RAND_0_1 ((double)rand()) / RAND_MAX

void framebufferSizeCallback(GLFWwindow *window, int width, int height) {
    glViewport(10, 10, width-10, height-10);
}  

void processInput(GLFWwindow *window) {
	if(glfwGetKey(window, GLFW_KEY_ESCAPE) == GLFW_PRESS) {
		glfwSetWindowShouldClose(window, true);
	}
}

void renderGraphics(GLFWwindow *window) {

}

void gracefulExit(int code) {
	glfwTerminate();
	exit(code);
}

std::string readFile(std::string path) {
	std::ifstream file(path);
	if (!file.is_open())
		std::cerr << "File " << path << " was not found." << std::endl;

	std::string content = "";
	std::string buffer = "";

	while(getline(file, buffer)) {
		content += buffer + "\n";
	}

	content += '\0';
	file.close();
	return content;
}

void updateVertices(double (&vertices)[24]) {
	// if (RAND_0_1 < 0.99f) return; 
	double a[] = {
		-1.0f, 1.0f, 0.0f,  RAND_0_1, RAND_0_1, RAND_0_1,    // upper left
    	-1.0f, -1.0f, 0.0f,  RAND_0_1, RAND_0_1, RAND_0_1,   // bottom left
     	1.0f,  1.0f, 0.0f,  RAND_0_1, RAND_0_1, RAND_0_1,    // upper right
		1.0f, -1.0f, 0.0f,  RAND_0_1, RAND_0_1, RAND_0_1     // bottom right
	};

	// vertices = a;
	memcpy(vertices, a, sizeof(a));
}

unsigned int prepareShaders() {
	unsigned int vertexShader = glCreateShader(GL_VERTEX_SHADER);
	unsigned int fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);

	std::string vertexShaderSourceString = readFile("../shaders/shader.vert");
	std::string fragmentShaderSourceString = readFile("../shaders/shader.frag");

	const char *vertexShaderSource = vertexShaderSourceString.c_str();
	const char *fragmentShaderSource = fragmentShaderSourceString.c_str();

	glShaderSource(vertexShader, 1, &vertexShaderSource, NULL);
	glCompileShader(vertexShader);

	int success;
	char log[512];
	glGetShaderiv(vertexShader, GL_COMPILE_STATUS, &success);
	if (!success) {
		glGetShaderInfoLog(vertexShader, 512, NULL, log);
		std::cerr << "Could not compile vertex shader: " << log << std::endl;
		gracefulExit(-1);
	}
	glShaderSource(fragmentShader, 1, &fragmentShaderSource, NULL);
	glCompileShader(fragmentShader);

	glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, &success);
	if (!success) {
		glGetShaderInfoLog(fragmentShader, 512, NULL, log);
		std::cerr << "Could not compile fragment shader: " << log << std::endl;
		gracefulExit(-1);
	}

	unsigned int shaderProgram;
	shaderProgram = glCreateProgram();
	glAttachShader(shaderProgram, vertexShader);
	glAttachShader(shaderProgram, fragmentShader);
	glLinkProgram(shaderProgram);

	glGetProgramiv(shaderProgram, GL_LINK_STATUS, &success);
	if(!success) {
    	glGetProgramInfoLog(shaderProgram, 512, NULL, log);
		std::cerr << "Could not link program: " << log << std::endl;
		gracefulExit(-1);
	}

	glDeleteShader(vertexShader);
	glDeleteShader(fragmentShader);
	return shaderProgram;
}

int main () {
	Coil c = Coil(2025, 1);

	// CoilSegment cs = CoilSegment(10, 0, false);
	for (auto segment : c.getSegments()) {
		// segment.printVertices();
	}
	// cs.printVertices();
	return 0;
// 	srand(time(0));
// 	glfwInit();
// 	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
// 	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
// 	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
// #ifdef __APPLE__
// 	glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
// #endif

// 	GLFWwindow *window = glfwCreateWindow(800, 600, "TimeCoil", NULL, NULL);
// 	if (window == NULL) {
// 		std::cerr << "Failed to create GLFW window." << std::endl;
// 		gracefulExit(-1);
// 	}

// 	glfwMakeContextCurrent(window);
// 	glfwSetFramebufferSizeCallback(window, framebufferSizeCallback);

// 	if (!gladLoadGLLoader((GLADloadproc)glfwGetProcAddress)) {
// 		std::cerr << "Failed to initialize GLAD." << std::endl;
// 		return -1;
// 	}

// 	unsigned int shaderProgram = prepareShaders();

// 	double *vertices = cs.getVertices();
// 	unsigned int amountOfVerticies = cs.getVerticesLength();
// 	std::cout << amountOfVerticies << std::endl;
// 	// double vertices[] = {
// 	// 	-1.0f, -1.0f, 0.0f,  1.0f, 0.0f, 0.0f,    // upper left
//     // 	-0.5f, -0.5f, 0.0f,  0.0f, 1.0f, 0.0f,   // bottom left
//     //  	0.5f,  0.5f, 0.0f,  1.0f, 0.0f, 0.0f,    // upper right
// 	// 	1.0f, 1.0f, 0.0f,  0.0f, 1.0f, 0.0f     // bottom right
// 	// };
// 	// unsigned int indices[] = {
// 	// 	1, 2, 3,
// 	// 	1, 2, 0
// 	// };
// 	unsigned int VBO, VAO, EBO;
// 	glGenVertexArrays(1, &VAO);
// 	glGenBuffers(1, &VBO);
// 	// glGenBuffers(1, &EBO);

// 	glBindVertexArray(VAO);

// 	glBindBuffer(GL_ARRAY_BUFFER, VBO);
// 	// glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, EBO);

// 	glBufferData(GL_ARRAY_BUFFER, amountOfVerticies, vertices, GL_STATIC_DRAW);
// 	// glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices, GL_STATIC_DRAW);

// 	glVertexAttribPointer(0, 3, GL_DOUBLE, GL_FALSE, 6 * sizeof(double), (void*)0);
// 	glEnableVertexAttribArray(0);

// 	glVertexAttribPointer(1, 3, GL_DOUBLE, GL_FALSE, 6 * sizeof(double), (void*) (3 * sizeof(double)));
// 	glEnableVertexAttribArray(1);

// 	glBindVertexArray(0);
	
// 	while(!glfwWindowShouldClose(window)) {
// 		processInput(window);

// 		glClearColor(0.2f, 0.3f, 0.3f, 1.0f);
// 		glClear(GL_COLOR_BUFFER_BIT);
	
// 		glUseProgram(shaderProgram);

// 		// double time = glfwGetTime();
// 		// double greenValue = sin(time);
		

// 		// int vertexColorLocation = glGetUniformLocation(shaderProgram, "MyColor");
// 		// glUniform4f(vertexColorLocation, 0, greenValue, 0, 1);
// 		glBindVertexArray(VAO);

// 		// updateVertices(vertices);
// 		glBufferData(GL_ARRAY_BUFFER, amountOfVerticies, vertices, GL_DYNAMIC_DRAW);
// 		// glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
// 		glDrawArrays(GL_LINE_STRIP_ADJACENCY, -30, amountOfVerticies);
		

// 		glBindVertexArray(0);
// 	    glfwSwapBuffers(window);
// 	    glfwPollEvents();
// 	}

// 	glDeleteVertexArrays(1, &VAO);
//     glDeleteBuffers(1, &VBO);
//     glDeleteProgram(shaderProgram);
// 	gracefulExit(0);
}
